#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "PID.h"
#include "type_def.h"
#include "debug_rt.h"
#include "communication.h"
#include "computation_graph.h"
#include "timer.h"




#define ReadBit(data, position) ( (data>>y) & 1)
#define SetBit(data,position) data |= (1 << y)
#define ClearBit(data,y) data &= ~(1 << y)



/*
    Initially sets it so that all debug values are zero
*/
SensorRTFlags_t * initializeFlags(void){

    SensorRTFlags_t * flags = malloc(sizeof(SensorRTFlags_t));
#if 0
    flags -> imuflags = calloc(sizeof(IMUFlags_t));
    flags -> optflowflags = calloc(sizeof(OptFlowFlags_t));
    flags -> lidarflags = calloc(sizeof(lidarFlags_t));
    flags -> errorFlags = calloc(sizeof(SensorErrorFlags_t));
#endif
}

void freeFlags(SensorRTFlags_t * flags){
#if 0
    free(flags -> imuflags);
    free(flags -> optflowflags); 
    free(flags -> lidarflags);
    free(flags -> errorFlags);
    free(flags);
#endif
}

/*
Uses the payload from the configuration packet (it's in the flag parameter) to set what should be sent
Size of float = 32 bits

As of now many bits are empty and wont be processed
See RTSensorPacketDocumentation.md for which flags each bit of the config data corresponds to

It returns the size of the packet that will be sent, so the payload is correctly determined
*/

int process_configuration_packet(float configData, SensorRTFlags_t * flags)
{

    int size = 0;
#if 0
    flags -> lidarflags -> quadHeight = ReadBit(31); size += ReadBit(31); 
    flags -> optflowflags ->  x_flow = ReadBit(30); size += 2 * ReadBit(30); 
    flags -> optflowflags ->  y_flow = ReadBit(29); size += 2 * ReadBit(29); 
    flags -> optflowflags ->  x_filter = ReadBit(28); size += 2 * ReadBit(28); 
    flags -> optflowflags ->  y_filter = ReadBit(27); size += 2 * ReadBit(27); 
    flags -> optflowflags ->  x_velocity = ReadBit(26); size += 2 * ReadBit(26); 
    flags -> optflowflags ->  y_velocity = ReadBit(25); size += 2 * ReadBit(25); 
    flags -> imuflags -> gyro_x = ReadBit(24); size += ReadBit(24); 
    flags -> imuflags -> gyro_y = ReadBit(23); size += ReadBit(23); 
    flags -> imuflags -> gyro_z = ReadBit(22); size += ReadBit(22); 
    flags -> imuflags -> acc_x = ReadBit(21); size += ReadBit(21); 
    flags -> imuflags -> acc_y = ReadBit(20); size += ReadBit(20); 
    flags -> imuflags -> acc_z = ReadBit(19); size += ReadBit(19); 
    flags -> imuflags -> mag_x = ReadBit(18); size += ReadBit(18); 
    flags -> imuflags -> mag_y = ReadBit(17); size += ReadBit(17); 
    flags -> imuflags -> mag_z = ReadBit(16); size += ReadBit(16); 
    flags -> errorflags -> lidar = ReadBit(15); size += ReadBit(15); 
    flags -> errorflags -> consec_lidar = ReadBit(14); size += ReadBit(14); 
    flags -> errorflags -> optFlow = ReadBit(13); size += ReadBit(13); 
    flags -> errorflags -> consec_optFlow = ReadBit(12); size += ReadBit(12); 
    flags -> errorflags -> imu = ReadBit(11); size += ReadBit(11); 
    flags -> errorflags -> consec_imu = ReadBit(10); size += ReadBit(10); 
#endif
    return size;
}

/*

typedef struct raw_sensor {

    gam_t gam;
    lidar_t lidar;
    px4flow_t optical_flow;

    // Structures to hold the current quad position & orientation
    // This is mostly unused?
    quadPosition_t currentQuadPosition;

} raw_sensor_t;


*/

int send_RT_data(struct CommDriver *comm, raw_sensor_t * data, SensorRTFlags_t * flags, int size)
{
    char * payload = malloc(sizeof(char) * size);
    int currentBytePosition = 0;
#if 0
    /*
    Since it's calloced (all zeros), a simple & operation should set this properly
    */
    if(flags -> lidarflags -> quadHeight){

        (*payload) &= ((data -> lidar -> distance_m) << (currentBytePosition * 8));
        currentBytePosition++; 
    } 

    if(flags -> optflowflags ->  x_flow) 
    {
        (*payload) &= ((data -> optical_flow -> ) << (currentBytePosition * 8));
        currentBytePosition += 2;
    }

    if(flags -> optflowflags ->  y_flow)
    {
        (*payload) &= ((data -> optical_flow -> ) << (currentBytePosition * 8));
        currentBytePosition += 2;
    }

    if(flags -> optflowflags ->  x_filter)
    {
        (*payload) &= ((data -> optical_flow -> ) << (currentBytePosition * 8));
        currentBytePosition += 2;
    }

    if(flags -> optflowflags ->  y_filter)
    {
        (*payload) &= ((data -> optical_flow -> ) << (currentBytePosition * 8));
        currentBytePosition += 2;
    }
    if(flags -> optflowflags ->  x_velocity)
    {
        (*payload) &= ((data -> optical_flow -> ) << (currentBytePosition * 8));
        currentBytePosition += 2;
    }

    if(flags -> optflowflags ->  y_velocity)
    {
        (*payload) &= ((data -> optical_flow -> ) << (currentBytePosition * 8));
        currentBytePosition += 2;
    }
    
    if(flags -> imuflags -> gyro_x)
    {
        (*payload) &= ((data -> gam -> ) << (currentBytePosition * 8));
        currentBytePosition++; 
    }

    if(flags -> imuflags -> gyro_y)
    {
        (*payload) &= ((data -> gam -> ) << (currentBytePosition * 8));
        currentBytePosition++; 
    }

    if(flags -> imuflags -> gyro_z)
    {
        (*payload) &= ((data -> gam -> ) << (currentBytePosition * 8));
        currentBytePosition++; 
    } 

    if(flags -> imuflags -> acc_x)
    {
        (*payload) &= ((data -> gam -> ) << (currentBytePosition * 8));
        currentBytePosition++; 
    }

    if(flags -> imuflags -> acc_y)
    {
        (*payload) &= ((data -> gam -> ) << (currentBytePosition * 8));
        currentBytePosition++; 
    }

    if(flags -> imuflags -> acc_z)
    {
        (*payload) &= ((data -> gam -> ) << (currentBytePosition * 8));
        currentBytePosition++; 
    }

    if(flags -> imuflags -> mag_x)
    {
        (*payload) &= ((data -> gam -> ) << (currentBytePosition * 8));
        currentBytePosition++; 
    }

    if(flags -> imuflags -> mag_y)
    {
        (*payload) &= ((data -> gam -> ) << (currentBytePosition * 8));
        currentBytePosition++; 
    }

    if(flags -> imuflags -> mag_z)
    {
        (*payload) &= ((data -> gam -> ) << (currentBytePosition * 8));
        currentBytePosition++; 
    }
    
    if(flags -> errorflags -> lidar)
    {
        (*payload) &= ((data -> lidar -> error -> errorCount) << (currentBytePosition * 8));
        currentBytePosition++; 
    }

    if(flags -> errorflags -> consec_lidar)
    {
        (*payload) &= ((data -> lidar -> consErrorCount -> errorCount) << (currentBytePosition * 8));
        currentBytePosition++; 
    }
    
    if(flags -> errorflags -> optFlow)
    {
        (*payload) &= ((data -> optical_flow -> error -> errorCount) << (currentBytePosition * 8));
        currentBytePosition++; 
    }

    if(flags -> errorflags -> consec_optFlow)
    {
        (*payload) &= ((data -> optical_flow -> consErrorCount -> errorCount) << (currentBytePosition * 8));
        currentBytePosition++; 
    }

    if(flags -> errorflags -> imu)
    {
        (*payload) &= ((data -> gam -> error -> errorCount) << (currentBytePosition * 8));
        currentBytePosition++;
    }
    
    if(flags -> errorflags -> consec_imu)
    {
        (*payload) &= ((data -> gam -> consErrorCount -> errorCount) << (currentBytePosition * 8));
        currentBytePosition++; 
    }


#endif
    int bytes_sent = send_data(comm->uart, SEND_RT_ID, 0, (u8*)payload, strlen(payload));
    free(payload);
    return bytes_sent; 
}
